#include <exception>
#include <QLibrary>
#include <QDebug>
#include <QDir>

// Заголовочные файлы ДТО9
#include "ircpprn.h"
#include "dto_errors.h"
#include "dto_const.h"

static const bool USE_SHOW_PROPERTIES = false;

#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
#  include <QTextCodec>

static class TrInit
{
public:
    TrInit()
    {
        QTextCodec::setCodecForTr(QTextCodec::codecForName("utf-8"));
        QTextCodec::setCodecForCStrings(QTextCodec::codecForName("utf-8"));
        QTextCodec::setCodecForLocale(QTextCodec::codecForName("utf-8"));
    }
} g_TrInit;
#endif

class Exception : public std::exception {
public:
    Exception(const QString &msg)
        : m_message(msg)
    {
    }

    virtual ~Exception() throw()
    {
    }

    virtual const char* what() const throw()
    {
        static QByteArray rawMessage = m_message.toUtf8();
        return rawMessage.data();
    }

private:
    QString m_message;
};

void checkError(TED::IDTOBase *iface)
{
    if(!iface)
        throw Exception("Invalid interface");

    int rc = EC_OK;
    iface->get_ResultCode(&rc);
    if(rc < 0)
    {
        QString resultDescription, badParamDescription;
        QVector<wchar_t> v(256);
        int size = iface->get_ResultDescription(&v[0], v.size());
        if (size <= 0)
            throw Exception("get_ResultDescription error");
        if (size > v.size())
        {
            v.clear();
            v.resize(size + 1);
            iface->get_ResultDescription(&v[0], v.size());
        }
        resultDescription = QString::fromWCharArray(&v[0]);
        if(rc == EC_INVALID_PARAM) {
            QVector<wchar_t> v(256);
            int size = iface->get_BadParamDescription(&v[0], v.size());
            if (size <= 0)
                throw Exception("get_BadParamDescription error");
            if (size > v.size())
            {
                v.clear();
                v.resize(size + 1);
                iface->get_ResultDescription(&v[0], v.size());
            }
            badParamDescription = QString::fromWCharArray(&v[0]);
        }

        if(badParamDescription.isEmpty())
            throw Exception(QObject::trUtf8("[%1] %2")
                            .arg(rc)
                            .arg(resultDescription));
        else
            throw Exception(QObject::trUtf8("[%1] %2 (%3)")
                            .arg(rc)
                            .arg(resultDescription)
                            .arg(badParamDescription));
    }
}

int main()
{
#if defined(Q_OS_LINUX)
    QLibrary lib("../bins/linux-x64/librcpprn.so");
    if(!lib.load())
    {
        lib.setFileName("../bins/linux-x86/librcpprn.so");
        lib.load();
    }
#elif defined(Q_OS_WIN32)
    QLibrary lib("../bins/nt-x86-mingw/rcpprn.dll");
    lib.load();
#else
#   error "Unknown OS"
#endif

    if(!lib.isLoaded())
    {
        qDebug() << qPrintable(QObject::trUtf8("Не удалось загрузить библиотеку rcpprn - [%1]")
                               .arg(lib.errorString()));
        return 1;
    }

    CreateRcpPrnInterfacePtr create = (CreateRcpPrnInterfacePtr) lib.resolve("CreateRcpPrnInterface");
    ReleaseRcpPrnInterfacePtr release = (ReleaseRcpPrnInterfacePtr) lib.resolve("ReleaseRcpPrnInterface");

    TED::RcpPrn::IRcpPrn *ircpprn = create(DTO_IRCPPRN_VER1);
    if(!ircpprn)
    {
        qDebug() << qPrintable("Cannot create RcpPrn object");
        return 1;
    }

    try
    {
        // Выставляем рабочий каталог. В нем дПЧ будет искать требуемые ему библиотеки.
        QFileInfo finfo(lib.fileName());
        if(ircpprn->put_DeviceSingleSetting(S_SEARCHDIR, finfo.absolutePath().toStdWString().c_str()) < 0)
            checkError(ircpprn);
        if(ircpprn->ApplySingleSettings() < 0)
            checkError(ircpprn);

        // Для настройки драйвера можно вызвать графическое окно настроек
        if(USE_SHOW_PROPERTIES)
        {
            if(ircpprn->ShowProperties() < 0)
                checkError(ircpprn);
        }
        // Или настроить без него
        else
        {
            // COM17
            //if(ircpprn->put_DeviceSingleSetting(S_PORT, 17) < 0)
                //checkError(ircpprn);
            if(ircpprn->put_DeviceSingleSetting(S_PORT, SV_PORT_TTY) < 0)
                checkError(ircpprn);
            if(ircpprn->put_DeviceSingleSetting(S_DEVICEFILE, L"ttyACM0") < 0)
                checkError(ircpprn);
            if(ircpprn->put_DeviceSingleSetting(S_BAUDRATE, 115200) < 0)
                checkError(ircpprn);
            if(ircpprn->put_DeviceSingleSetting(S_MODEL, TED::RcpPrn::ModelPP7000) < 0)
                checkError(ircpprn);
            if(ircpprn->ApplySingleSettings() < 0)
                checkError(ircpprn);
        }

        // Активируем дПЧ
        if (ircpprn->put_DeviceEnabled(1) < 0)
            checkError(ircpprn);

        // Добавляем строки в буфер заданий
        if(ircpprn->put_Caption(L"Строка1") < 0)
            checkError(ircpprn);
        if(ircpprn->put_Alignment(TED::RcpPrn::AlignmentLeft) < 0)
            checkError(ircpprn);
        if(ircpprn->AddText() < 0)
            checkError(ircpprn);

        if(ircpprn->put_Caption(L"Строка2") < 0)
           checkError(ircpprn);
        if(ircpprn->put_Alignment(TED::RcpPrn::AlignmentCenter) < 0)
           checkError(ircpprn);
        if(ircpprn->AddText() < 0)
            checkError(ircpprn);

        if(ircpprn->put_Caption(L"Строка3") < 0)
           checkError(ircpprn);
        if(ircpprn->put_Alignment(TED::RcpPrn::AlignmentRight) < 0)
           checkError(ircpprn);
        if(ircpprn->AddText() < 0)
            checkError(ircpprn);

        // Добавляем ШК в буфер заданий
        if(ircpprn->put_BarcodeType(TED::RcpPrn::BarcodeEAN13) < 0)
            checkError(ircpprn);
        if(ircpprn->put_BarcodeText(L"123456789012") < 0)
            checkError(ircpprn);
        if(ircpprn->put_BarcodePrintText(1) < 0)
            checkError(ircpprn);
        if(ircpprn->put_BarcodeControlCode(0) < 0)
            checkError(ircpprn);
        if(ircpprn->put_Alignment(TED::RcpPrn::AlignmentCenter) < 0)
            checkError(ircpprn);
        if(ircpprn->AddBarCode() < 0)
            checkError(ircpprn);

        // Печатаем буфер
        if(ircpprn->PrintAllTasks() < 0)
            checkError(ircpprn);
    }
    catch(const std::exception &ex)
    {
        qDebug() << qPrintable(ex.what());
        release(&ircpprn);
        return 1;
    }

    release(&ircpprn);
    return 0;
}

